function getNewHolidayIdValue() {
    var idValue = 0;
    var baseNode = Property.getNode('values/holiday');
    if (baseNode == null)
        return idValue;
    var oArrayNode = baseNode.getChildren();
    for ( var iIndex = 0; iIndex < oArrayNode.length; iIndex++) {
        var oChildNode = oArrayNode[iIndex];
        if (oChildNode.getChild('id') != null)
            if (oChildNode.getChild('id').getValue() != null)
                if (parseInt("" + oChildNode.getChild('id').getValue(),10) >= idValue)
                    idValue = parseInt("" + oChildNode.getChild('id').getValue(),10) + 1;
    }
    return idValue;
}

function storeHolidayParameter(path, value) {
    try 
    {
        lInfo.logln("storeHolidayParameter Path " + path  + " value " + value + " type " + typeof(value));
        Property.setProperty('values/holiday/' + path, value);
        Property.setFlag('values/holiday/' + path, 'ARCHIVE', true);
    }
    catch (e)
    {
        lWarn.logln("Fehler beim storeHolidayParameter " + e + " Path " + path  + " value " + value);
    }        
}

function storeHolidayValue(sValue) {
    try {
        var result =  {};
        var oParameterObject = JSON.parse(unescape(sValue));

        var idValue = null;
        if (oParameterObject.id != null) {
            idValue =  isNaN(parseInt("" + oParameterObject.id)) ? getNewHolidayIdValue() : parseInt("" + oParameterObject.id); 
        } else {
            idValue = getNewHolidayIdValue();
        }
        result.id = idValue;
        var baseNode = Property.getNode('values/holiday');
        if (baseNode != null)
            baseNode.removeChild(idValue);

        //the unescape already ensures to be string
        storeHolidayParameter(idValue + '/name', Utf8.encode(unescape(oParameterObject.name)));
        storeHolidayParameter(idValue + '/id', idValue);
        var startDate = Math.floor((new Date(oParameterObject.von)).getTime()/1000);
        var endDate = Math.floor((new Date(oParameterObject.bis)).getTime()/1000);
        if (!isNaN(startDate) && !isNaN(endDate)) {
            storeHolidayParameter(idValue + '/von', startDate);
            storeHolidayParameter(idValue + '/bis', endDate);
            result.success = true;
        } else {
            //we could not store all data successfully so we cleanup
            baseNode.removeChild(idValue);
            result.success = false;  
        }
    
        var evt = new Event("system-addon-timed-events.config", {actions:'updateTimer'});
        evt.raise();
        return result;
        
    } catch (e) {
        lWarn.logln("Fehler beim Speichern: " + e);
        for ( var i in e) {
            l.logln(i + ': ' + e[i]);
        }
    }
}

/**
 * Removes the temporaryHoliday property node.
 * @returns {Boolean} true if node has been removed, false otherwise. 
 */
function deleteTempHolidayValues() {
    try {
        var baseNode = Property.getNode('/scripts/system-addon-presence-simulator');
        if (baseNode != null) {
            baseNode.removeChild('temporaryHoliday');;
        }
        return true;
    } catch (e) {
        lWarn.logln('exception in presence-simulator-config:deleteTempHolidayValues ' + e);
        return false;
    }
}

/**
 * Stores a new temp holiday entry in property tree.
 * If startTime is empty, current time will be used. active state will be set to false will be 
 * calculated right afterwards in checkHoliday
 * @param {string} startTime if empty string provided, current time will be set
 * @param {string} returnTime if empty string is provided no returnTime will be set
 * @returns {boolean} true if succeeded false otherwise
 */
function storeTempHolidayValues(startTime, returnTime) {
    var success = false;
    try {
        lInfo.logln('storeTempHolidayValues startTime: ' + startTime + ' returnTime: '+ returnTime);
        if (startTime !== '') {
            success = storeStartTime(startTime);
        } else {
            var oDate = new Date(); //now
            startTime = Math.floor(oDate.getTime()/1000);
            success = storeStartTime(startTime);
        }
        if (success) {
            if ((returnTime !== '') && (startTime <= returnTime)){
                storeReturnTime(returnTime);
            }
            setTempHolidayState(false);
        }
    } catch (e) {
        lWarn.logln('exception in presence-simulator-config:storeTempHolidayValues ' + e);        
    }
    return success;
}

/**
 * sets temp holiday values. If temp holiday already exist only return time will be set, otherwise a 
 * new tempHoliday will be set up
 * @param {sring} sStartTime
 * @param {sring} sReturnTime
 * @returns {boolean} true if successful, false otherwise
 */
function setTempHoliday(sStartTime, sReturnTime) {
    var success = false;
    try {
        lInfo.logln('setTempHoliday startTime: ' + sStartTime + ' returnTime: '+ sReturnTime);
        var baseNode = Property.getNode('temporaryHoliday');
        if (baseNode !== null) {
            if ((Property.getNode('temporaryHoliday/from').getValue()) <= sReturnTime) {
                //temp holiday already exists, store only new return time
                success = storeReturnTime(sReturnTime);
            }
        } else { 
            //setup a new temp holiday
            success = storeTempHolidayValues(sStartTime, sReturnTime);
        }
    } catch (e) {
        lWarn.logln('exception in presence-simulator-config:setTempHoliday ' + e);        
    }
    return success;
}

/**
 * Sets the return time of a temp holiday. 
 * If no temp holiday exists or is not active a new one which starts immediately will be set up.
 * @param {type} sReturnTime
 * @returns {boolean} true if succeeded, false otherwise
 */
function setReturnTime(sReturnTime) {
    var success = false;
    try {
        var baseNode = Property.getNode('temporaryHoliday');
        if (baseNode !== null) {
            if ((baseNode.getChild('active')) && (baseNode.getChild('active').getValue() == true)) {
                if ((Property.getNode('temporaryHoliday/from').getValue()) <= sReturnTime) {
                    success = storeReturnTime(sReturnTime);
                    return success;
                }
            } else {
                deleteTempHolidayValues();
            }       
        } 
        var oDate = new Date(); //now
        success = storeTempHolidayValues(Math.floor(oDate.getTime()/1000), sReturnTime);
    } catch (e) {
        lWarn.logln('exception in presence-simulator-config:setReturnTime ' + e);        
    }
    return success;
}

/**
 * Stores the start time of temporary holiday to property tree
 * @param {string} startTime time stamp in sec since 1.1.1970. Holiday will be terminated at this time
 * @returns {boolean} true if successful, false otherwise
 */
function storeStartTime(startTime) {
    var success = true;
    try {
        startTime = parseInt("" + startTime);
        //var startDate = Math.floor((new Date(startTime)).getTime()/1000);
        if (!isNaN(startTime)) {
            lInfo.logln('storeStartTime: converted in seconds startTime: ' + startTime);        
            Property.setProperty('temporaryHoliday/from', startTime);
            Property.setFlag('temporaryHoliday/from', 'ARCHIVE', true);
        } else {
            lWarn.logln('storeStartTime: converted start time is NaN and will not be stored');
            success = false;
        }
    } catch (e) {
        lWarn.logln('exception in presence-simulator-config:storeStartTime ' + e);
        success = false;
    }
    return success;
}

/**
 * Stores the return time of temporary holiday to property tree
 * @param {string} returnTime time stamp in sec since 1.1.1970. Holiday will be terminated at this time
 * @returns {boolean} true if successful, false otherwise
 */
function storeReturnTime(returnTime) {
    var success = true;
    try {
        returnTime = parseInt("" + returnTime);
        //var returnDate = Math.floor((new Date(returnTime)).getTime()/1000);
        if (!isNaN(returnTime)) {
            lInfo.logln('storeReturnTime: converted in seconds returnTime: ' + returnTime);    
            Property.setProperty('temporaryHoliday/until', returnTime);
            Property.setFlag('temporaryHoliday/until', 'ARCHIVE', true);
        } else {
            lWarn.logln('storeReturnTime: converted return time is NaN');            
            success = false;
        }
    } catch (e) {
        lWarn.logln('exception in presence-simulator-config:storeReturnTime ' + e);        
        success = false;
    }
    return success;
}

/**
 * Stores the temp holiday state value
 * @param {boolean} bActive
 * @returns {undefined}
 */
function setTempHolidayState(bActive) {
    try {
        var baseNode = Property.getNode('temporaryHoliday');
        lInfo.logln('setTempHolidayState baseNode: ' + baseNode);
        if (baseNode != null) {
            Property.setProperty('temporaryHoliday/active', bActive);
            Property.setFlag('temporaryHoliday/active', 'ARCHIVE', true);    
        }
    } catch (e) {
        lWarn.logln('exception in presence-simulator-config:setTempHolidayState ' + e);        
    }
}

if (raisedEvent.name == 'system-addon-presence-simulator.holiday-config') {
	Property.load();
    if (raisedEvent.parameter.actions == 'save') {
        lInfo.logln("PS-Config Holiday param value " + raisedEvent.parameter.value);       
        var result = storeHolidayValue(raisedEvent.parameter.value);
        var id = result.id;
        Property.store();   
        
        var evt = new Event("system-addon-presence-simulator",  {actions:'checkHoliday'});
        evt.raise();
        var evt2;
        if (result.success) {
            evt2 = new Event("system-addon-presence-simulator.saved", {id:id, complete:'success'});
        } else {
            evt2 = new Event("system-addon-presence-simulator.saved", {id:id, complete:'failed'});
        }
        evt2.raise();
    }
    if (raisedEvent.parameter.actions == 'delete') {
        lInfo.logln("PS-Config Holiday delete value " + raisedEvent.parameter.id);
                
        var oIDObject=JSON.parse(unescape(raisedEvent.parameter.id));
        if (typeof(oIDObject)=='number')
            oIDObject=[oIDObject];
        
        for (var sKey in oIDObject) {
	        var baseNode = Property.getNode('values/holiday');
	        if (baseNode != null)
	            baseNode.removeChild(oIDObject[sKey]);
        }
        
        Property.store();

        var evt = new Event("system-addon-presence-simulator",  {actions:'checkHoliday'});
        evt.raise();

        var evt2 = new Event("system-addon-presence-simulator.saved",  {id:raisedEvent.parameter.id});
        evt2.raise();
    }
}

if (raisedEvent.name == 'system-addon-presence-simulator.temporary-holiday') {
    Property.load();
    try {
        lInfo.logln('system-addon-presence-simulator.temporary-holiday event received');
        lInfo.logln('-- action: ' + raisedEvent.parameter.actions);
        lInfo.logln('-- startTime: ' + raisedEvent.parameter.startTime);
        lInfo.logln('-- returnTime: ' + raisedEvent.parameter.returnTime);
        var success = true;
        switch (raisedEvent.parameter.actions) {
            case 'startTemporaryHoliday' :
                {
                    var sStarTime = (raisedEvent.parameter.startTime) ? String(raisedEvent.parameter.startTime) : ''; 
                    var sReturnTime = (raisedEvent.parameter.returnTime) ? String(raisedEvent.parameter.returnTime) : ''; 
                    success = setTempHoliday(sStarTime, sReturnTime);
                }
                break;
            case 'setReturnTime' :
                {
                    success = setReturnTime(raisedEvent.parameter.returnTime);
                }
                break;
            case 'endTemporaryHoliday' :
                {
                    success = deleteTempHolidayValues();
                }
                break;
            default: 
                {
                    lWarn.logln('system-addon-presence-simulator.temporary-holiday with unexpected action');
                    success = false;
                }
        }
        if (success) {
            Property.store();
            lInfo.logln('-send checkHoliday event');
            var evt = new Event("system-addon-presence-simulator",  {actions:'checkHoliday'});
            evt.raise();
            lInfo.logln('-send saved event');
            var evt2 = new Event("system-addon-presence-simulator.saved", {complete:'success'});
            evt2.raise();            
        } else {
            lInfo.logln('-send failed saved event');
            var evt2 = new Event("system-addon-presence-simulator.saved", {complete:'failed'});
            evt2.raise();            
        }
    } catch (e) {
        lWarn.logln('exception on handling event system-addon-presence-simulator.temporary-holiday' + e);
        var evt = new Event("system-addon-presence-simulator.saved", {complete:'failed'});
        evt.raise();
    }
}
    
